@@ -27,7 +27,7 @@ def _build_parser():
     """
     Parser builder
 
-    :returns: instanceof ArgumentParser
+    :return: instanceof ArgumentParser
     :rtype: ```ArgumentParser```
     """
     parser = ArgumentParser(
@@ -160,6 +160,11 @@ def _build_parser():
         type=str,
         dest="function_names",
     )
+    sync_parser.add_argument(
+        "--no-word-wrap",
+        help="Whether word-wrap is disabled (on emission). None enables word-wrap. Defaults to None.",
+        action="store_true",
+    )
     sync_parser.add_argument(
         "--truth",
         help=(
@@ -233,6 +238,11 @@ def _build_parser():
             " function"
         ),
     )
+    gen_parser.add_argument(
+        "--no-word-wrap",
+        help="Whether word-wrap is disabled (on emission). None enables word-wrap. Defaults to None.",
+        action="store_true",
+    )
     gen_parser.add_argument(
         "--decorator",
         help="List of decorators.",
@@ -338,6 +348,11 @@ def _build_parser():
         dest="type_annotations",
         action="store_false",
     )
+    doctrans_parser_group.add_argument(
+        "--no-word-wrap",
+        help="Whether word-wrap is disabled (on emission). None enables word-wrap. Defaults to None.",
+        action="store_true",
+    )
 
     #########
     # exmod #
@@ -361,8 +376,14 @@ def _build_parser():
         help="What type to generate.",
         choices=parse_emit_types,
         required=True,
+        dest="emit_name",
         action="append",
     )
+    exmod_parser.add_argument(
+        "--no-word-wrap",
+        help="Whether word-wrap is disabled (on emission). None enables word-wrap. Defaults to None.",
+        action="store_true",
+    )
     exmod_parser.add_argument(
         "--blacklist",
         help="Modules/FQN to omit. If unspecified will emit all (unless whitelist).",
@@ -398,7 +419,7 @@ def main(cli_argv=None, return_args=False):
     :param return_args: Primarily use is for tests. Returns the args rather than executing anything.
     :type return_args: ```bool```
 
-    :returns: the args if `return_args`, else None
+    :return: the args if `return_args`, else None
     :rtype: ```Optional[Namespace]```
     """
     _parser = _build_parser()
@@ -408,7 +429,11 @@ def main(cli_argv=None, return_args=False):
     if command == "sync":
         args = Namespace(
             **{
-                k: v if k == "truth" or isinstance(v, list) or v is None else [v]
+                k: v
+                if k in frozenset(("truth", "no_word_wrap"))
+                or isinstance(v, list)
+                or v is None
+                else [v]
                 for k, v in args_dict.items()
             }
         )
@@ -480,26 +505,15 @@ def main(cli_argv=None, return_args=False):
             )
         )
     elif command == "openapi":
-        openapi_bulk(
-            app_name=args.app_name,
-            model_paths=args.model_paths,
-            routes_paths=args.routes_paths,
-        )
+        openapi_bulk(**args_dict)
     elif command == "doctrans":
         require_file_existent(_parser, args.filename, name="filename")
-        doctrans(
-            filename=args.filename,
-            docstring_format=args.format,
-            type_annotations=args.type_annotations,
-        )
+        args_dict["docstring_format"] = args_dict.pop("format")
+        doctrans(**args_dict)
     elif command == "exmod":
         exmod(
-            module=args.module,
-            emit_name=args.emit,
-            blacklist=args.blacklist,
-            whitelist=args.whitelist,
-            output_directory=args.output_directory,
-            dry_run=args.dry_run,
+            mock_imports=False,  # This option is really only useful for tests IMHO
+            **args_dict
         )
 
 
