@@ -13,16 +13,19 @@ import ast
 from ast import AST, Constant, NameConstant
 from collections import OrderedDict
 from copy import deepcopy
-from enum import Enum
 from functools import partial
 from itertools import chain, takewhile
-from operator import attrgetter, contains, eq, le
+from operator import attrgetter, eq, le
 from typing import Dict, List, Tuple
 
 from cdd.ast_utils import NoneStr, get_value
 from cdd.defaults_utils import _remove_default_from_param, needs_quoting
-from cdd.docstring_utils import ARG_TOKENS, RETURN_TOKENS, TOKENS
-from cdd.emit import to_code
+from cdd.docstring_utils import (
+    ARG_TOKENS,
+    RETURN_TOKENS,
+    Style,
+    derive_docstring_format,
+)
 from cdd.emitter_utils import interpolate_defaults
 from cdd.pure_utils import (
     code_quoted,
@@ -35,17 +38,7 @@ from cdd.pure_utils import (
     unquote,
     update_d,
 )
-
-
-class Style(Enum):
-    """
-    Simple enum taken from the docstring_parser codebase
-    """
-
-    rest = 1
-    google = 2
-    numpydoc = 3
-    auto = 255
+from cdd.source_transformer import to_code
 
 
 def parse_docstring(
@@ -80,7 +73,7 @@ def parse_docstring(
     :param emit_default_doc: Whether help/docstring should include 'With default' text
     :type emit_default_doc: ```bool```
 
-    :returns: a dictionary of form
+    :return: a dictionary of form
         {  "name": Optional[str],
            "type": Optional[str],
            "doc": Optional[str],
@@ -140,25 +133,6 @@ def parse_docstring(
     return ir
 
 
-def derive_docstring_format(docstring):
-    """
-    Infer the docstring format of the provided docstring
-
-    :param docstring: the docstring
-    :type docstring: ```Optional[str]```
-
-    :returns: the style of docstring
-    :rtype: ```Literal['rest', 'numpydoc', 'google']```
-    """
-    if docstring is None or any(map(partial(contains, docstring), TOKENS.rest)):
-        style = Style.rest
-    elif any(map(partial(contains, docstring), TOKENS.google)):
-        style = Style.google
-    else:
-        style = Style.numpydoc
-    return style
-
-
 def _scan_phase(docstring, parse_original_whitespace=False, style=Style.rest):
     """
     Scanner phase. Lexical analysis; to some degree…
@@ -172,7 +146,7 @@ def _scan_phase(docstring, parse_original_whitespace=False, style=Style.rest):
     :param style: the style of docstring
     :type style: ```Style```
 
-    :returns: List with each element a tuple of (whether value is a token, value)
+    :return: List with each element a tuple of (whether value is a token, value)
     :rtype: ```Union[Dict[str, str], List[Tuple[bool, str]]]```
     """
     arg_tokens, return_tokens = map(attrgetter(style.name), (ARG_TOKENS, RETURN_TOKENS))
@@ -208,7 +182,7 @@ def _scan_phase_numpydoc_and_google(
     :param style: the style of docstring
     :type style: ```Style```
 
-    :returns: List with each element a tuple of (whether value is a token, value)
+    :return: List with each element a tuple of (whether value is a token, value)
     :rtype: ```Dict[str, str]```
     """
     white_spacer = (
@@ -225,11 +199,12 @@ def _scan_phase_numpydoc_and_google(
     # ^ Dict[Union[Literal["doc"], arg_tokens, return_tokens], List[dict]]
 
     # First doc, if present
-    _start_idx, _end_idx, _found = location_within(docstring, arg_tokens)
+    _start_idx, _end_idx, _found = (
+        lambda _loc: location_within(docstring, return_tokens)
+        if _loc[0] == -1
+        else _loc
+    )(location_within(docstring, arg_tokens))
     # _leading_whitespace = "".join(takewhile(str.isspace, docstring[:_start_idx][::-1]))
-    if _start_idx == -1:
-        # Return type no args?
-        _start_idx, _end_idx, _found = location_within(docstring, return_tokens)
 
     if _start_idx > -1:
         namespace = _found
@@ -270,35 +245,40 @@ def _scan_phase_numpydoc_and_google(
                 scanned[return_tokens[0]] = docstring_lines[
                     line_no + 2 : line_no + 3 + next_smallest_indent
                 ]
+                if len(scanned[return_tokens[0]]) > 1 and not scanned[return_tokens[0]][
+                    0
+                ].endswith(":"):
+                    scanned[return_tokens[0]] = ["\n".join(scanned[return_tokens[0]])]
+
                 scanned_afterward = docstring_lines[
                     line_no + 3 + next_smallest_indent :
                 ]
             else:
                 scanned_afterward = docstring_lines[line_no + 1 :]
-                if (
-                    len(scanned_afterward) > 1
-                    and scanned_afterward[0] == return_tokens[0]
-                ):
-                    return_indent = count_iter_items(
-                        takewhile(str.isspace, scanned_afterward[1])
-                    )
-                    next_smallest_indent = count_iter_items(
-                        takewhile(
-                            partial(le, return_indent),
-                            map(
-                                lambda l: count_iter_items(takewhile(str.isspace, l)),
-                                scanned_afterward[2:],
-                            ),
-                        )
-                    )
-                    scanned[return_tokens[0]] = scanned_afterward[
-                        1 : next_smallest_indent + 2
-                    ]
-                    scanned_afterward = (
-                        None
-                        if next_smallest_indent == 0
-                        else scanned_afterward[next_smallest_indent + 2 :]
-                    )
+                # if (
+                #     len(scanned_afterward) > 1
+                #     and scanned_afterward[0] == return_tokens[0]
+                # ):
+                #     return_indent = count_iter_items(
+                #         takewhile(str.isspace, scanned_afterward[1])
+                #     )
+                #     next_smallest_indent = count_iter_items(
+                #         takewhile(
+                #             partial(le, return_indent),
+                #             map(
+                #                 lambda l: count_iter_items(takewhile(str.isspace, l)),
+                #                 scanned_afterward[2:],
+                #             ),
+                #         )
+                #     )
+                #     scanned[return_tokens[0]] = scanned_afterward[
+                #         1 : next_smallest_indent + 2
+                #     ]
+                #     scanned_afterward = (
+                #         None
+                #         if next_smallest_indent == 0
+                #         else scanned_afterward[next_smallest_indent + 2 :]
+                #     )
 
             if scanned_afterward:
                 scanned["scanned_afterward"] = scanned_afterward
@@ -340,18 +320,41 @@ def _return_parse_phase_numpydoc_and_google(return_tokens, scanned, stacker, sty
     :param style: the style of docstring
     :type style: ```Style```
 
-    :returns: Whatever is left of `stacker`. This function may also set the return key of the `scanned`
+    :return: Whatever is left of `stacker`. This function may also set the return key of the `scanned`
     :rtype: ```List[List[str]]```
     """
-    rev_return_token = return_tokens[0].splitlines()[::-1]
+    _return_tokens = return_tokens[0].splitlines()
+    rev_return_token = _return_tokens[::-1]
     rng = range(len(stacker) - 1, -1, -1)
-    if style is Style.numpydoc:
+    if style is Style.numpydoc or style is Style.google:
         for i in rng:
             if i - 1 > 0 and stacker[i] + stacker[i - 1] == rev_return_token:
                 scanned[return_tokens[0]] = stacker[i + 1 :]
                 stacker = stacker[: i - 1]
                 break
 
+    if not scanned.get(return_tokens[0], False) and all(
+        (
+            style is Style.numpydoc or style is Style.google,
+            len(scanned.get("scanned_afterward", "")),
+        )
+    ):
+        found, fst_idx, snd_idx = False, None, None
+        for idx, line in enumerate(scanned["scanned_afterward"]):
+            if line.lstrip().startswith(_return_tokens[0]):
+                found, fst_idx = True, idx
+            elif found is True:
+                if line.isspace():
+                    snd_idx = idx
+                    break
+                else:
+                    scanned[_return_tokens[0]].append(line)
+        if fst_idx is not None:
+            del scanned["scanned_afterward"][fst_idx:snd_idx]
+            scanned[_return_tokens[0]] = ["\n".join(scanned[_return_tokens[0]])]
+            if sum(map(len, map(str.strip, scanned["scanned_afterward"]))) == 0:
+                del scanned["scanned_afterward"]
+
     return stacker
 
 
@@ -368,7 +371,7 @@ def _scan_phase_rest(docstring, arg_tokens, return_tokens):
     :param return_tokens: Valid tokens like `":rtype:"`
     :type return_tokens: ```Tuple[str]```
 
-    :returns: List with each element a tuple of (whether value is a token, value)
+    :return: List with each element a tuple of (whether value is a token, value)
     :rtype: ```List[Tuple[bool, str]]```
     """
 
@@ -480,7 +483,7 @@ def _set_name_and_type(param, infer_type, word_wrap):
     :param word_wrap: Whether to word-wrap. Set `DOCTRANS_LINE_LENGTH` to configure length.
     :type word_wrap: ```bool```
 
-    :returns: Name, dict with keys: 'typ', 'doc', 'default'
+    :return: Name, dict with keys: 'typ', 'doc', 'default'
     :rtype: ```Tuple[str, dict]```
     """
     name, _param = param
@@ -643,7 +646,7 @@ def _parse_phase_numpydoc_and_google(
             :param scan: Scanned input
             :type scan: ```List[str]```
 
-            :returns: dict with keys: 'name', 'typ', 'doc'
+            :return: dict with keys: 'name', 'typ', 'doc'
             :rtype: ```dict```
             """
             name, _, typ = scan[0].partition(":")
@@ -668,7 +671,7 @@ def _parse_phase_numpydoc_and_google(
             :param partitioned: Prep-partitioned `scan`, if given doesn't partition on `scan`, just uses this
             :type partitioned: ```Optional[Tuple[str, str, str]]```
 
-            :returns: dict of shape {'name': ..., 'typ': ..., 'doc': ..., 'default': ..., 'required': ... }
+            :return: dict of shape {'name': ..., 'typ': ..., 'doc': ..., 'default': ..., 'required': ... }
             :rtype: ```dict```
             """
             offset = next(idx for idx, ch in enumerate(scan[0]) if ch == ":")
@@ -735,7 +738,7 @@ def _parse_phase_numpydoc_and_google(
         :param name_param: Name, dict with keys: 'typ', 'doc', 'default'
         :type name_param: ```Tuple[str, dict]```
 
-        :returns: Name, dict with keys: 'typ', 'doc', 'default'
+        :return: Name, dict with keys: 'typ', 'doc', 'default'
         :rtype: ```Tuple[str, dict]```
         """
         name, param = interpolate_defaults(
@@ -971,7 +974,7 @@ def _set_param_values(input_str, val, sw=":type"):
     :param sw: Startswith condition
     :type sw: ```str```
 
-    :returns: Properly derived key and [potentially modified] value
+    :return: Properly derived key and [potentially modified] value
     :rtype: Tuple[Literal['doc', 'typ'], str]
     """
     return (
@@ -981,4 +984,4 @@ def _set_param_values(input_str, val, sw=":type"):
     )
 
 
-__all__ = ["derive_docstring_format", "parse_docstring"]
+__all__ = ["parse_docstring"]
