@@ -12,11 +12,14 @@ from ast import (
     walk,
 )
 from collections import OrderedDict
+from copy import deepcopy
 from operator import attrgetter
 
 from cdd import emit, parse
 from cdd.ast_utils import (
+    annotate_ancestry,
     find_in_ast,
+    maybe_type_comment,
     set_arg,
     set_docstring,
     to_annotation,
@@ -50,7 +53,12 @@ class DocTrans(NodeTransformer):
     """
 
     def __init__(
-        self, docstring_format, type_annotations, existing_type_annotations, whole_ast
+        self,
+        docstring_format,
+        word_wrap,
+        type_annotations,
+        existing_type_annotations,
+        whole_ast,
     ):
         """
         Transform the docstrings found to intended docstring_format, potentially manipulating type annotations also
@@ -58,6 +66,9 @@ class DocTrans(NodeTransformer):
         :param docstring_format: Format of docstring
         :type docstring_format: ```Literal['rest', 'numpydoc', 'google']```
 
+        :param word_wrap: Whether to word-wrap. Set `DOCTRANS_LINE_LENGTH` to configure length.
+        :type word_wrap: ```bool```
+
         :param type_annotations: True to have type annotations (3.6+), False to place in docstring
         :type type_annotations: ```bool```
 
@@ -67,10 +78,16 @@ class DocTrans(NodeTransformer):
         :param whole_ast: The entire input AST, useful for lookups by location
         :type whole_ast: ```AST``
         """
+
         self.docstring_format = docstring_format
+        self.word_wrap = word_wrap
         self.type_annotations = type_annotations
         self.existing_type_annotations = existing_type_annotations
-        self.whole_ast = whole_ast
+        if not hasattr(whole_ast, "_location"):
+            self.whole_ast = deepcopy(whole_ast)
+            annotate_ancestry(self.whole_ast)
+        else:
+            self.whole_ast = whole_ast
         self.memoized = {}
 
     # def generic_visit(self, node):
@@ -80,13 +97,13 @@ class DocTrans(NodeTransformer):
     #     :param node: The AST node
     #     :type node: ```AST```
     #
-    #     :returns: Potentially changed AST node
+    #     :return: Potentially changed AST node
     #     :rtype: ```AST```
     #     """
     #     is_func, doc_str = isinstance(node, (AsyncFunctionDef, FunctionDef)), None
     #     if is_func or isinstance(node, ClassDef):
     #         node, doc_str = self._handle_node_with_docstring(node)
-    #         doc_str = ast.get_docstring(node)
+    #         doc_str = ast.get_docstring(node, clean=True)
     #     if is_func:
     #         node = self._handle_function(node, doc_str)
     #     return super(DocTrans, self).generic_visit(node)
@@ -98,19 +115,19 @@ class DocTrans(NodeTransformer):
         :param node: AnnAssign
         :type node: ```AnnAssign```
 
-        :returns: `AnnAssign` if `type_annotations` and type found else `Assign`
+        :return: `AnnAssign` if `type_annotations` and type found else `Assign`
         :rtype: ```Union[AnnAssign, Assign]```
         """
         if self.type_annotations:
             node.annotation = self._get_ass_typ(node)
-            node.type_comment = None
+            setattr(node, "type_comment", None)
             return node
 
         return Assign(
             targets=[node.target],
             value=node.value,
-            type_comment=to_type_comment(node.annotation),
             lineno=None,
+            type_comment=to_type_comment(node.annotation),
         )
 
     def visit_Assign(self, node):
@@ -120,25 +137,28 @@ class DocTrans(NodeTransformer):
         :param node: Assign
         :type node: ```Assign```
 
-        :returns: `AnnAssign` if `type_annotations` and type found else `Assign`
+        :return: `AnnAssign` if `type_annotations` and type found else `Assign`
         :rtype: ```Union[Assign, AnnAssign]```
         """
         typ = self._get_ass_typ(node)
-        if self.type_annotations:
+        annotation = (
+            None if not self.type_annotations or typ is None else to_annotation(typ)
+        )
+        if annotation:
             assert len(node.targets) == 1
             return AnnAssign(
                 annotation=to_annotation(typ),
                 value=node.value,
-                type_comment=None,
                 lineno=None,
                 simple=1,
                 target=node.targets[0],
                 expr=None,
                 expr_target=None,
                 expr_annotation=None,
+                **maybe_type_comment
             )
         else:
-            node.type_comment = typ
+            setattr(node, "type_comment", typ)
         return node
 
     def visit_Module(self, node):
@@ -148,11 +168,11 @@ class DocTrans(NodeTransformer):
         :param node: Module
         :type node: ```Module```
 
-        :returns: Potentially changed Module
+        :return: Potentially changed Module
         :rtype: ```Module```
         """
         # Clean might be wrong if the header is a license or other long-spiel documentation
-        doc_str = get_docstring(node, clean=True)
+        doc_str = get_docstring(node, clean=self.word_wrap is True)
         empty = doc_str is None
         if not empty:
             set_docstring("\n{doc_str}\n".format(doc_str=doc_str), empty, node)
@@ -166,10 +186,10 @@ class DocTrans(NodeTransformer):
         :param node: FunctionDef
         :type node: ```FunctionDef```
 
-        :returns: Potentially changed FunctionDef
+        :return: Potentially changed FunctionDef
         :rtype: ```FunctionDef```
         """
-        return self._handle_function(node, get_docstring(node))
+        return self._handle_function(node, get_docstring(node, clean=True))
 
     def _get_ass_typ(self, node):
         """
@@ -178,13 +198,16 @@ class DocTrans(NodeTransformer):
         :param node: Assignment
         :type node: ```Union[Assign, AnnAssign]```
 
-        :returns: The type of the assignment, e.g., `int`
+        :return: The type of the assignment, e.g., `int`
         :rtype: ```Optional[str]```
         """
         name, typ_dict = (
-            (node.targets[0], {"typ": node.type_comment})
+            (node.targets[0], {"typ": getattr(node, "type_comment", None)})
             if isinstance(node, Assign)
-            else (node.target, {"typ": node.annotation or node.type_comment})
+            else (
+                node.target,
+                {"typ": node.annotation or getattr(node, "type_comment", None)},
+            )
         )
         if not hasattr(node, "_location"):
             return typ_dict["typ"]
@@ -200,7 +223,7 @@ class DocTrans(NodeTransformer):
                             lambda doc_str: None
                             if doc_str is None
                             else parse.docstring(doc_str)
-                        )(get_docstring(parent))
+                        )(get_docstring(parent, clean=True))
                         if isinstance(parent, (ClassDef, AsyncFunctionDef, FunctionDef))
                         else {"params": OrderedDict()}
                     )
@@ -223,14 +246,14 @@ class DocTrans(NodeTransformer):
         :param doc_str: The docstring
         :type doc_str: ```Optional[str]```
 
-        :returns: Same type as input with args, returns, and docstring potentially modified
+        :return: Same type as input with args, returns, and docstring potentially modified
         :rtype: ```Union[AsyncFunctionDef, FunctionDef]```
         """
         ir = parse_docstring(doc_str)
         ir_merge(ir, parse.function(node))
         ir["name"] = node.name
         indent_level = max(
-            len(node._location) - 1, 1
+            len(node._location), 1
         )  # function docstrings always have at least 1 indent level
         _doc_str = emit.docstring(
             ir,
@@ -238,8 +261,9 @@ class DocTrans(NodeTransformer):
             emit_default_doc=False,
             docstring_format=self.docstring_format,
             indent_level=indent_level,
+            word_wrap=self.word_wrap,
         )
-        if _doc_str.isspace():
+        if not _doc_str or _doc_str.isspace():
             if doc_str is not None:
                 del node.body[0]
         else:
@@ -254,6 +278,7 @@ class DocTrans(NodeTransformer):
                             annotation=to_annotation(
                                 _arg.annotation
                                 if _arg.annotation is not None
+                                or _arg.arg in frozenset(("self", "cls"))
                                 else ir["params"][_arg.arg].get("typ")
                             ),
                         ),
@@ -281,13 +306,13 @@ def clear_annotation(node):
     :param node: AST node
     :type node: ```AST```
 
-    :returns: AST node with annotations and type_comments set to `None`
+    :return: AST node with annotations and type_comments set to `None`
     :rtype: ```AST```
     """
     if getattr(node, "annotation", None) is not None:
         node.annotation = None
     if getattr(node, "type_comment", None) is not None:
-        node.type_comment = None
+        setattr(node, "type_comment", None)
     return node
 
 
