@@ -47,7 +47,7 @@ def identity(*args, **kwargs):
     :param args: Any values
     :type args: ```Tuple[Any]```
 
-    :returns: the input value
+    :return: the input value
     :rtype: ```Any```
     """
     return args[0] if len(args) == 1 else args
@@ -107,7 +107,7 @@ def pluralise(singular):
     :param singular: Non plural
     :type singular: ```str```
 
-    :returns: Plural version
+    :return: Plural version
     :rtype: ```str```
     """
     if not singular:
@@ -140,6 +140,27 @@ def pluralise(singular):
     return root + suffix
 
 
+# def previous_line_range(s, countdown_from=None):
+#     """
+#     Get previous line from a multiline string
+#
+#     :param s: Multiline string
+#     :type s: ```str```
+#
+#     :param countdown_from: Index to start looking down from; len(s) otherwise
+#     :type countdown_from: ```Optional[int]```
+#
+#     :return: Previous line range if found else None
+#     :rtype: ```Optional[Tuple[int, int]]```
+#     """
+#     if countdown_from is None:
+#         countdown_from = len(s)
+#     for i in range(countdown_from - 1, 0, -1):
+#         if s[i] == "\n":
+#             return i + 1, countdown_from
+#     return None
+
+
 def deindent(s, level=None, sep=tab):
     """
     Remove all indentation from the input string, or `level`(s) of indent if specified
@@ -153,7 +174,7 @@ def deindent(s, level=None, sep=tab):
     :param sep: Separator (usually `tab`)
     :type sep: ```str```
 
-    :returns: Deindented string
+    :return: Deindented string
     :rtype: ```AnyStr```
     """
     if level is None:
@@ -166,7 +187,7 @@ def deindent(s, level=None, sep=tab):
             :param line: The line to dedent
             :type line: ```AnyStr```
 
-            :returns: Dedented line
+            :return: Dedented line
             :rtype: ```AnyStr```
             """
             return line[len(sep) :] if line.startswith(sep) else line
@@ -187,7 +208,7 @@ def reindent(s, indent_level=1, join_on="\n"):
     :param join_on: What to join on, e.g., '\n'
     :type join_on: ```str```
 
-    :returns: Reindented string
+    :return: Reindented string
     :rtype: ```AnyStr```
     """
     return join_on.join(
@@ -200,6 +221,17 @@ def reindent(s, indent_level=1, join_on="\n"):
     ).replace(tab, "", 1)
 
 
+def strip_starting(line, str_to_strip=tab):
+    """
+    :param line: Input string
+    :type line: ```AnyStr```
+
+    :param str_to_strip: Removes only this (once… so not `str.lstrip`) from the start
+    :type str_to_strip: ```str```
+    """
+    return line[len(str_to_strip) :] if line.startswith(str_to_strip) else line
+
+
 def indent_all_but_first(s, indent_level=1, wipe_indents=False, sep=tab):
     """
     Indent all lines except the first one
@@ -216,7 +248,7 @@ def indent_all_but_first(s, indent_level=1, wipe_indents=False, sep=tab):
     :param sep: Separator (usually `tab`)
     :type sep: ```str```
 
-    :returns: input string indented (except first line)
+    :return: input string indented (except first line)
     :rtype: ```str```
     """
     lines = indent(deindent(s) if wipe_indents else s, sep * abs(indent_level)).split(
@@ -242,7 +274,7 @@ def multiline(s, quote_with=("'", "'")):
     :param quote_with: What to quote with
     :type quote_with: ```Tuple[str, str]```
 
-    :returns: multine input string
+    :return: multine input string
     :rtype: ```str```
     """
     return "{}{}".format(
@@ -265,7 +297,7 @@ def sanitise(s):
     :param s: Input string
     :type s: ```str```
 
-    :returns: input string with '_' append if it's a keyword else input string
+    :return: input string with '_' append if it's a keyword else input string
     :rtype: ```str```
     """
     return "{}_".format(s) if iskeyword(s) else s
@@ -281,7 +313,7 @@ def strip_split(param, sep):
     :param sep: Separator
     :type sep: ```str```
 
-    :returns: Iterator of each element of the hierarchy
+    :return: Iterator of each element of the hierarchy
     :rtype: ```Iterator[str, ...]```
     """
     return map(str.strip, param.split(sep))
@@ -294,7 +326,7 @@ def unquote(input_str):
     :param input_str: Input string
     :type input_str: ```Optional[str]```
 
-    :returns: Unquoted string
+    :return: Unquoted string
     :rtype: ```Optional[str]```
     """
     if (
@@ -321,7 +353,7 @@ def quote(s, mark='"'):
     :param mark: Quote mark to wrap with
     :type mark: ```str```
 
-    :returns: Quoted string or input (if input is not str)
+    :return: Quoted string or input (if input is not str)
     :rtype: ```Union[str, float, complex, int, None]```
     """
     very_simple_types = type(None), int, float, complex
@@ -359,7 +391,7 @@ def assert_equal(a, b, cmp=eq):
     :param cmp: comparator function
     :type cmp: ```Callable[[a, b], bool]```
 
-    :returns: True if equal, otherwise raises `AssertionError`
+    :return: True if equal, otherwise raises `AssertionError`
     :rtype: ```Literal[True]```
     """
     if not cmp(a, b):
@@ -380,7 +412,7 @@ def update_d(d, arg=None, **kwargs):
     :param kwargs: keyword args to update with
     :type kwargs: ```**kwargs```
 
-    :returns: Updated dict
+    :return: Updated dict
     :rtype: ```dict```
     """
     if arg:
@@ -400,7 +432,7 @@ def lstrip_namespace(s, namespaces):
     :param namespaces: namespaces to strip
     :type namespaces: ```Union[List[str], Tuple[str], Generator[str], Iterator[str]]```
 
-    :returns: `.lstrip`ped input (potentially just the original!)
+    :return: `.lstrip`ped input (potentially just the original!)
     :rtype: ```AnyStr```
     """
     for namespace in namespaces:
@@ -419,7 +451,7 @@ def diff(input_obj, op):
     :param op: The operation to run
     :type op: ```Callable[[Any], Any]```
 
-    :returns: length of difference, response of operated input
+    :return: length of difference, response of operated input
     :rtype: ```Tuple[int, Any]```
     """
     input_len = len(
@@ -448,7 +480,7 @@ def blockwise(t, size=2, fillvalue=None):
     :param fillvalue: What to use to "pair" with if uneven
     :type fillvalue: ```Any```
 
-    :returns: iterator with iterators inside of block size
+    :return: iterator with iterators inside of block size
     :rtype: ```Iterator```
     """
     return zip_longest(*[iter(t)] * abs(size), fillvalue=fillvalue)
@@ -468,7 +500,7 @@ def location_within(container, iterable, cmp=eq):
     :param cmp: Comparator to check input against
     :type cmp: ```Callable[[str, str], bool]```
 
-    :returns: (Start index iff found else -1, End index iff found else -1, subset iff found else None)
+    :return: (Start index iff found else -1, End index iff found else -1, subset iff found else None)
     :rtype: ```Tuple[int, int, Optional[Any]]```
     """
     if not hasattr(container, "__len__"):
@@ -491,16 +523,83 @@ def location_within(container, iterable, cmp=eq):
     return -1, -1, None
 
 
-BUILTIN_TYPES = (
-    frozenset(
-        chain.from_iterable(
-            map(
-                lambda s: (s, "typing.{}".format(s), "_extensions.{}".format(s)),
-                filter(lambda s: s[0].isupper() and not s.isupper(), dir(typing)),
-            )
+BUILTIN_TYPES = frozenset(
+    chain.from_iterable(
+        (
+            chain.from_iterable(
+                map(
+                    lambda s: (s, "typing.{}".format(s), "_extensions.{}".format(s)),
+                    filter(lambda s: s[0].isupper() and not s.isupper(), dir(typing)),
+                )
+            ),
+            (
+                "int",
+                "float",
+                "complex",
+                "list",
+                "tuple",
+                "str",
+                "bytes",
+                "bytearray",
+                "memoryview",
+                "set",
+                "frozenset",
+                "dict",
+            ),
         )
     )
-    | frozenset(("int", "float", "str", "dict", "list", "tuple"))
+)
+
+DUNDERS = frozenset(
+    filter(
+        rpartial(str.startswith, "__"),
+        frozenset(
+            chain.from_iterable(
+                (
+                    # https://docs.python.org/3/library/stdtypes.html
+                    chain.from_iterable(
+                        map(
+                            dir,
+                            (
+                                int,
+                                float,
+                                complex,
+                                list,
+                                tuple,
+                                range,
+                                str,
+                                bytes,
+                                bytearray,
+                                memoryview,
+                                set,
+                                frozenset,
+                                dict,
+                                type,
+                                None,
+                                Ellipsis,
+                                NotImplemented,
+                                object,
+                            ),
+                        )
+                    ),
+                    # https://docs.python.org/3/library/functions.html#dir
+                    dir(),
+                    # https://docs.python.org/3/library/stdtypes.html#special-attributes
+                    (
+                        "__dict__",
+                        "__class__",
+                        "__bases__",
+                        "__name__",
+                        "__qualname__",
+                        "__mro__",
+                        "__subclasses__",
+                        # https://docs.python.org/3/reference/datamodel.html#slots
+                        "__slots__",
+                    ),
+                )
+            )
+        ),
+    )
 )
 
 INIT_FILENAME = "__init__{extsep}py".format(extsep=extsep)
@@ -515,7 +614,7 @@ def code_quoted(s):
     :param s: The input
     :type s: ```Any```
 
-    :returns: Whether the input is code quoted
+    :return: Whether the input is code quoted
     :rtype: ```bool```
     """
     return (
@@ -531,7 +630,7 @@ def count_iter_items(iterable):
     :param iterable: An iterable
     :type iterable: ```Iterable```
 
-    :returns: Number of items in iterable
+    :return: Number of items in iterable
     :rtype: ```int```
     """
     counter = count()
@@ -558,7 +657,7 @@ def get_module(name, package=None, extra_symbols=None):
     :param extra_symbols: Dictionary of extra symbols to use if `importlib.import_module` fails
     :type extra_symbols: ```Optional[dict]```
 
-    :returns: Module
+    :return: Module
     :rtype: ```Module```
     """
     try:
@@ -577,6 +676,23 @@ def get_module(name, package=None, extra_symbols=None):
             raise
 
 
+def has_nl(s, func):
+    """
+    Check if str endswith a newline (possibly separated by other whitespace)
+
+    :param s: Input string
+    :type s: ```str```
+
+    :param func: `str.partition` or `str.rpartition` function (or something similar)
+    :type func: ```Callable[[str], Tuple[str,str,str]]```
+
+    :return: Whether s endswith a newline (possibly separated by other whitespace)
+    :rtype: ```bool```
+    """
+    start, maybe_nl, end = partial(func, "\n")(s)
+    return maybe_nl == "\n" and start.isspace()
+
+
 def paren_wrap_code(code):
     """
     The new builtin AST unparser adds extra parentheses, so match that behaviour on older versions
@@ -584,7 +700,7 @@ def paren_wrap_code(code):
     :param code: Source code string
     :type code: ```str```
 
-    :returns: Potentially parenthetically wrapped input
+    :return: Potentially parenthetically wrapped input
     :rtype: ```str```
     """
     return (
@@ -601,7 +717,7 @@ def filename_from_mod_or_filename(mod_or_filename):
     :param mod_or_filename: Module name or filename
     :type mod_or_filename: ```str```
 
-    :returns: Filename
+    :return: Filename
     :rtype: ```str```
     """
     filename = type("", tuple(), {"origin": mod_or_filename})
@@ -625,7 +741,7 @@ def emit_separating_tabs(s, indent_level=1, run_per_line=str.lstrip):
     :param run_per_line: Run this function per line
     :type run_per_line: ```Callable[[str], str]```
 
-    :returns: Original string with a separating tab between paragraphs, & possibly addition indentation on other lines
+    :return: Original string with a separating tab between paragraphs, & possibly addition indentation on other lines
     :rtype: ```str```
     """
     sep = tab * indent_level
@@ -652,7 +768,7 @@ def set_attr(obj, key, val):
     :param val: A value
     :type val: ```Any```
 
-    :returns: The modified `obj`
+    :return: The modified `obj`
     :rtype: ```Any```
     """
     setattr(obj, key, val)
@@ -674,7 +790,7 @@ def set_item(obj, key, val):
     :param val: A value
     :type val: ```Any```
 
-    :returns: The modified `obj`
+    :return: The modified `obj`
     :rtype: ```Any```
     """
     obj[key] = val
@@ -690,7 +806,7 @@ def no_magic_dir2attr(p_object):
     :param p_object: Object
     :type p_object: ```Any```
 
-    :returns: Dict of name to attribute value
+    :return: Dict of name to attribute value
     :rtype: ```dict```
     """
     return {
@@ -700,8 +816,17 @@ def no_magic_dir2attr(p_object):
     }
 
 
+omit_whitespace = rpartial(str.translate, str.maketrans({" ": "", "\n": "", "\t": ""}))
+
+sanitise_emit_name = dict(
+    **{typ: typ for typ in ("function", "sqlalchemy", "sqlalchemy_table")},
+    **{"class": "class_", "argparse": "argparse_function"}
+).__getitem__
+
+
 __all__ = [
     "BUILTIN_TYPES",
+    "DUNDERS",
     "ENCODING",
     "INIT_FILENAME",
     "PY3_8",
@@ -711,28 +836,35 @@ __all__ = [
     "blockwise",
     "code_quoted",
     "count_iter_items",
+    "deindent",
     "diff",
     "emit_separating_tabs",
     "filename_from_mod_or_filename",
     "fill",
     "get_module",
+    "has_nl",
     "identity",
     "indent_all_but_first",
     "location_within",
     "lstrip_namespace",
+    "multiline",
     "no_magic_dir2attr",
     "none_types",
+    "omit_whitespace",
     "paren_wrap_code",
     "pluralise",
     "pp",
+    # "previous_line_range",
     "quote",
     "reindent",
     "rpartial",
     "sanitise",
+    "sanitise_emit_name",
     "set_attr",
     "set_item",
     "simple_types",
     "strip_split",
+    "strip_starting",
     "tab",
     "unquote",
     "update_d",
